#include "TCanvas.h"
#include "TMath.h"
#include "TFile.h"
#include "TNtupleD.h"
#include "TH1D.h"
#include "TROOT.h"

#include <iomanip>
#include <sstream>
#include <algorithm>

using namespace std;


int GetRandom(vector<double> &cumPSI)
{
    double r     = gRandom->Uniform();
    auto   where = find_if(cumPSI.begin(), cumPSI.end(), [&](double y) { return y > r; } );

    return distance(cumPSI.begin(), where);
}

TH1D *FitNc(const char* fname)
{
  auto   fp   = new TFile(fname);
  auto   hNcp = static_cast<TNtupleD *>(gROOT->FindObject("hNc"));

  int    nmax =  1000.;
  int    nmin =   200.;
  double xmin =   nmin;
  double xmax =   nmax;

  stringstream cmdstr;
  cmdstr << "nc>>hnc(" << nmax << "," << 0.5 << "," << nmax + 0.5 << ")" << ends;
  hNcp->Draw(cmdstr.str().data());
  auto   hp = static_cast<TH1D *>(gROOT->FindObject("hnc"));

  hp->Scale(1./hp->Integral());
#if 1
  double av = 0;
  for (int i=1; i<nmax; i++) av += hp->GetBinContent(i)*i;
  cerr << "av =" << av << endl;
#endif

  auto   funp = new TF1("fun","[0]*exp(-[2]*x)/pow(x,[1])",xmin,xmax);
  funp->SetParameter(0,1.);
  funp->SetParameter(1,2.);
  funp->SetParameter(2,0.);

  double norm = hp->GetBinContent(nmin)/funp->Eval(xmin);
  funp->SetParameter(0,norm);

  hp->Fit("fun","","goff",xmin,xmax);
#if 0
  for (int i=nmin; i<nmax; i++) hp->SetBinContent(i,funp->Eval(i));
#endif

  static std::vector<double> cumPSI;
  double sum = 0.;
  for (int i=1; i<nmax; i++) {
    sum += hp->GetBinContent(i);
    cumPSI.push_back(sum);
  }
  for (int i=0; i<nmax-1; i++) {
    cumPSI[i] /= sum;
    //cerr << "cumPSI[" << i << "]=" << setprecision(14) << cumPSI[i] << endl;
  }

  int nev = 10000000;
  double av2 = 0;
  auto h2p = new TH1D("h2","",nmax,0.5,nmax+0.5);
  for (int i=0; i<nev; i++) {
    double m = GetRandom(cumPSI);
    //cerr << "m=" << m << endl;
    h2p->Fill(m+1);
    av2 += m+1;
  }
  av2 /= nev;
  cerr << "av2 = " << av2 << endl;
  h2p->Scale(1./h2p->Integral());
  //h2p->Draw();


  hp->Draw("same");

  TFile ofile("nc.root","recreate");
  hp->Write();
  ofile.Close();
  return hp;
}
