#include <vector>
#include <algorithm>
#include <iostream>
#include <iomanip>

#include <TRandom.h>
#include <TFile.h>
#include <TTree.h>
#include <TNtupleD.h>

using namespace std;

int GetRandom(vector<double> &cumPSI)
{
    double r     = gRandom->Uniform();
    auto   where = find_if(cumPSI.begin(), cumPSI.end(), [&](double y) { return y > r; } );

    return distance(cumPSI.begin(), where);
}

TH1D *scan(const char *fname   = "edep.electron.5e+09.root") 
{
  // ---------------
  // Open input file
  // ---------------
  cerr << "Infile: " << fname << endl;
  TFile &file = *new TFile(fname);
  cerr << "File opened" << endl;

  // ---------------
  // Set up TTree
  // ---------------
  TTree *tp = static_cast<TTree *>(file.Get("hNc"));
  double nc, ec;
  tp->SetBranchAddress("nc", &nc);
  tp->SetBranchAddress("ec", &ec);

  int nlist = tp->GetEntries();
  //cerr << "nlist = " << nlist << endl;

  // ---------------
  // Loop over hits
  // ---------------
  const int kNc = 3000;
  auto hp = new TH1D("hnc","",kNc,0.5,kNc+0.5);

  vector<double> prob(kNc);
  double ncsum = 0;
  double npt   = 0;
  for (int i=0; i<nlist; i++) {
    tp->GetEntry(i);
    int index = (int)(nc-0.5);
    if (nc > 0 && nc <= kNc) {
      prob[index] += 1.;
      ncsum += nc;
      hp->Fill(nc);
      npt++;
    }
  }
  double avnc = ncsum/npt;
  cerr << "avnc = " << avnc << endl;
#if 1
  int    nmax =  kNc;
  int    nmin =  200;
  double xmin = nmin;
  double xmax = nmax;
  auto   funp = new TF1("fun","[0]*exp(-[2]*x)/pow(x,[1])",xmin,xmax);
  funp->SetParameter(0,1.);
  funp->SetParameter(1,2.);
  funp->SetParameter(2,0.);

  double norm = hp->GetBinContent(nmin)/funp->Eval(xmin);
  funp->SetParameter(0,norm);

  hp->Fit("fun","","goff",xmin,xmax);
#if 1
  for (int i=nmin; i<nmax; i++) hp->SetBinContent(i,funp->Eval(i));
#endif
#endif

  for (int j=0; j<kNc; j++) prob[j] /= npt;

  static std::vector<double> cumPSI;
  double psum = 0;
  for (auto p : prob) {
    psum += p;
    cumPSI.push_back(psum);
  }

#if 0
  int nev = 1000000;
  double av2 = 0;
  auto h2p = new TH1D("h2","",kNc,0.5,kNc+0.5);
  for (int i=0; i<nev; i++) {
#if 0
    double m = GetRandom(cumPSI);
#else
    double m = hp->GetRandom()-1;
#endif
    //cerr << "m=" << m << endl;
    h2p->Fill(m+1);
    av2 += m+1;
  }
  av2 /= nev;
  cerr << "av2 = " << av2 << endl;
  h2p->Scale(1./h2p->Integral());

  hp->Scale(1./hp->Integral());
#if 0
  for (int j=0; j<kNc; j++) hp->SetBinContent(j+1,prob[j]);
#endif
  //hp->Draw();
  h2p->Draw("same");
#endif

  TFile ofile("nc.root","recreate");
  hp->Write();
  ofile.Close();
  return hp;
}
